/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.mahout.clustering.fuzzykmeans;

import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

import java.io.IOException;
import java.util.Iterator;

public class FuzzyKMeansCombiner extends MapReduceBase implements
    Reducer<Text, FuzzyKMeansInfo, Text, FuzzyKMeansInfo> {

  @Override
  public void reduce(Text key, Iterator<FuzzyKMeansInfo> values,
                     OutputCollector<Text, FuzzyKMeansInfo> output, Reporter reporter) throws IOException {
    SoftCluster cluster = new SoftCluster(key.toString().trim());
    while (values.hasNext()) {
      //String pointInfo = values.next().toString();
      FuzzyKMeansInfo info = values.next();

      if (info.getCombinerPass() == 0) // first time thru combiner
      {
        cluster.addPoint(info.getVector(), Math.pow(info.getProbability(), SoftCluster.getM()));
      } else {
        cluster.addPoints(info.getVector(), info.getProbability());
      }
      info.setCombinerPass(info.getCombinerPass() + 1);
    }
    //TODO: how do we pass along the combinerPass?  Or do we not need to?
    output.collect(key, new FuzzyKMeansInfo(cluster.getPointProbSum(), cluster.getWeightedPointTotal(), 1));
  }

  @Override
  public void configure(JobConf job) {
    super.configure(job);
    SoftCluster.configure(job);
  }

}
